/*
 * Group.java
 *
 * Created on July 2, 2006, 9:28 AM
 * 
 *  Virtual Fight. A programming game.
 *  Copyright (C) 2006  Aidin Gharibnavaz.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package virtualfight.group;

/**
 * @author Aidin Gharibnavaz
 *         (aidin.vf@gmail.com)
 */

import virtualfight.map.Map;

public class Group {
    
    private Charachter[] chs=new Charachter[5];
    private Integer ID;
    private Map map;
    
    public Group(int ID,Map map) {
        this.ID=ID;
        this.map=map;
        map.placeCharachters(this);
    }
    
    public void setCharLocation(int chNumber,int ylocation,int xlocation){
        chs[chNumber]=new Charachter(ylocation, xlocation);
    }
    
    public int changeCharDirection(int chNumber,String direction){
        if (chs[chNumber].getAllowMoving()){
            this.chs[chNumber].setDirection(direction);
            chs[chNumber].setAllowMoving(false);
            return 255;
        }
        return 120;
    }
    
    /*
     * move Charachter and return the result.
     * result is an int. result can be one of these:
     *
     *   225    | OK.
     *   111    | The destination is not empty.
     *   112    | Charachter wants to move out of screen.
     *   120    | This charachter can't move, because it moved before.
     */
    public int moveCharachter(int chNumber,String direction){
        //If this Charachter can't move, it return error and exit.
        //For example, if charachter died, or moved before.
        if (chs[chNumber].getAllowMoving()==false){
            //####################################
            System.out.println("charachter"+chNumber+"of group"+this.ID+" can't move");
            //####################################
            return 120;
        }
        //'where' is an array. it hold the direction we want to move to.
        int[] where;
        //'destinationLocation' is the location charachter wants to move to. First of all
        //we check whether it's empty or not. then we move charachter to it.
        int[] destinationLocation=new int[2];
        
        where=processDirection(direction);
        //#################################
        System.out.println("chs"+chNumber+":"+chs[chNumber].location[0]+","+chs[chNumber].location[1]);
        System.out.println("want to move "+direction);
        //################################
        destinationLocation[0]=chs[chNumber].location[0]+where[0];
        destinationLocation[1]=chs[chNumber].location[1]+where[1];
        
        //if 'destinationLocation' is out of screen.
        if ((destinationLocation[0]<0)||(destinationLocation[1]<0)
            ||(destinationLocation[0]>map.height-1)||
                (destinationLocation[1]>map.width-1)){
            //#################################
            System.out.println("out of Screen");
            //#################################
            return 112;
        //if 'destinationLocation' is not empty.
        }else if (!map.isEmpty(destinationLocation[0], destinationLocation[1])){
            //#################################
            System.out.println("destination is not Empty");
            //#################################
            return 111;
        //everything is OK, moving!
        }else{
            //################################
            System.out.println("OK, moving!");
            //################################
            map.charachterMoved(chs[chNumber].location[0], chs[chNumber].location[1],
                    destinationLocation[0], destinationLocation[1], this.ID);
            chs[chNumber].setLocation(destinationLocation[0], destinationLocation[1], direction);
            //This player is moved, so it can't move again.
            chs[chNumber].setAllowMoving(false);
            //#####################################
            System.out.println("chs"+chNumber+" moved to "+chs[chNumber].location[0]+","+chs[chNumber].location[1]);
            //#####################################
        }
        return 225;
    }
    
    /*
     * Get a direction, and return where we
     * must go.
     */
    private int[] processDirection(String direction){
        int[] where= new int[2];
        if (direction.equals("UP")){
            where[0]=-1;
            where[1]=0;
            return where;
        }
        if (direction.equals("LEFT")){
            where[0]=0;
            where[1]=-1;
            return where;
        }
        if (direction.equals("RIGHT")){
            where[0]=0;
            where[1]=1;
            return where;
        }
        if (direction.equals("DOWN")){
            where[0]=1;
            where[1]=0;
            return where;
        }
        return where;
    }
    
    //The charachter move is done.
    //We use it for 'NONE' command, without direction.
    public void chMoveDone(int chNumber){
        this.chs[chNumber].setAllowMoving(false);
    }
    
    /*
     * Return what a charachter can see.
     */
    public String whatDoYouSee(int chNumber){
        //################################
        System.out.println("Charachter"+chNumber+" Direction: "+chs[chNumber].getDirection());
        //################################
        return this.map.whatInSight(chs[chNumber].location[0],
                chs[chNumber].location[1], chs[chNumber].getDirection(),
                this.ID);
    }
    
    /*
     * Return Charachter location in String format.
     * When a client request a charachter position,
     * then we call this method, and send location
     * for the client.
     * See 'processClient' in Main.java
     */
    public String getCharachterLocation(int chNumber){
        return this.chs[chNumber].getLocation();
    }
    
    /*
     * Allow all of the charachters to move again.
     * For example when turn done.
     */
    public void resetTurns(){
        for(int i=0;i<5;i++){
            chs[i].setAllowMoving(true);
        }
    }
    
    /*
     * If all of the charachter of this group did
     * their move, return true.
     * We use it to check if all of the clients did
     * their move, then start another turn.
     */
    public boolean isAllDone(){
        for (int i=0;i<5;i++){
            if (chs[i].getAllowMoving()){
                return false;
            }
        }
        return true;
    }
    
    //Return the ID of this group.
    public Integer getID(){
        return this.ID;
    }
}
