/*
 * ClientHandler.java
 *
 * Created on July 4, 2006, 8:59 AM
 *
 *  Virtual Fight. A programming game.
 *  Copyright (C) 2006  Aidin Gharibnavaz.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package virtualfight.network;

/**
 * @author Aidin Gharibnavaz
 *         (aidin.vf@gmail.com)
 */
import virtualfight.VirtualFight;

import java.net.Socket;
import java.io.BufferedReader;
import java.io.PrintWriter;
import java.io.InputStreamReader;

public class ClientHandler extends Thread{
    
    Socket clientSock;
    String clientAddress;
    VirtualFight main;
    int ID;
    
    public ClientHandler(Socket s,String clientAddress,int ID,VirtualFight main) {
        this.clientSock=s;
        this.clientAddress=clientAddress;
        this.ID=ID;
        this.main=main;
        System.out.println("/------------------------------------------");
        System.out.println("/Client connection from "+this.clientAddress);
        System.out.println("/Using ID: "+this.ID);
        System.out.println("/------------------------------------------");
    }
    
    public void run(){
        try{
            BufferedReader in  = new BufferedReader( 
                new InputStreamReader(clientSock.getInputStream()));
            PrintWriter out = 
                new PrintWriter(clientSock.getOutputStream(), true);
            //until client is active, we process messages.
            //after that, we close the socket for this client.
            processClient(in,out);
            clientSock.close();
            System.out.println("/------------------------------------------");
            System.out.println("/Client (" + clientAddress + ") connection closed");
            System.out.println("/the ID was: "+this.ID);
            System.out.println("/------------------------------------------");
        }catch (Exception e){
            System.out.println(e);
        }
    }
    
    private void processClient(BufferedReader in, PrintWriter out){
        String line;
        boolean done = false;
        try {
            while (!done) {
                //We convert all of the messages to the upper case.
                //So it dosen't matter if receved messagess are in
                //upper or lower case.
                line = in.readLine().toUpperCase();
                System.out.println("message from client"+this.ID+":"+line);
                if (line.trim().equals("BYE")){
                    done = true;
                }else{
                    //send receved message to the main.
                    main.analyzeMessage(line.trim(),this.ID);
                }
            }
        }
        catch(Exception e){
            System.out.println(e);
        }
    }
    
    /*
     * Send 'message' to the client.
     * There is two versions of 'sendMessage',
     * one for String messages, and one for ints.
     */
    public void sendMessage(String message){
        try{
            PrintWriter output=
                    new PrintWriter(clientSock.getOutputStream(),true);
            output.println(message);
        }catch (Exception e){
            System.out.println(e);
        }
    }
    
    public void sendMessage(int message){
        try{
            PrintWriter output=
                    new PrintWriter(clientSock.getOutputStream(),true);
            output.println(message);
        }catch (Exception e){
            System.out.println(e);
        }
    }
}
